#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

class Reader
{
protected:
	FILE *f;
	int putBackCh;
public:
	Reader(FILE *F) : putBackCh(-1), f(F) { assert(f); }
	~Reader() { if (f) fclose(f); }
	void PutBack(char ch) { assert(putBackCh < 0); putBackCh = ch; }
	int GetCh() {
		if (putBackCh >= 0) { int ch = putBackCh; putBackCh = -1; return ch; }
		else { /*printf("!");*/ return fgetc(f); }}
	bool SkipWs(int& ch) { bool any = false; for ( ; ; ) {
		ch = GetCh();
//printf("<%d; %d>", any, ch);
		if (ch == EOF) return any;
		else if (! (isspace(ch) || ch == '\xa' || ch == '\xd')) { PutBack(ch); return any; }
		else any = true; } }
	bool SkipSpaces(int& ch) { bool any = false; for ( ; ; ) {
		ch = GetCh();
		if (ch == EOF) return any;
		else if (! (isspace(ch) && ch != '\xa' && ch != '\xd')) { PutBack(ch); return any; }
		else any = true; } }
	bool ReadInt(int& i, FILE *fRept, const int lineNo, const char *cesa);
	bool ReadString(char *dest, int maxChars, const char* extraStopChars); // stop at space or extraStopChars
	bool AtEof() { int ch = GetCh(); if (ch < 0) return true; else { PutBack(ch); return false; }}
	bool AtEol() { int ch = GetCh(); bool b = (ch == '\xa' || ch == '\xd');
		if (ch >= 0) PutBack(ch);
		return b; }
	bool ReadLn() {
		int ch = GetCh(); if (ch < 0) return false;
		if (ch == '\xa') return true;
		if (ch == '\xd') {
			ch = GetCh();
			if (ch >= 0 && ch != '\xa') PutBack(ch);
			return true; }
		PutBack(ch); return false; }
	void ReadLnA() { bool ok = ReadLn(); assert(ok); }
};

bool Reader::ReadString(char *dest, int maxChars, const char* extraStopChars)
{
	int nRead = 0;
	for ( ; ; )
	{
		int ch = GetCh();
		if (ch < 0 || isspace(ch) || strchr(extraStopChars, ch)) { dest[nRead] = 0; if (ch > 0) PutBack(ch); return true; }
		if (nRead == maxChars) { dest[nRead] = 0; return false; }
		dest[nRead++] = char(ch);
	}
}

bool Reader::ReadInt(int& i, FILE *fRept, const int lineNo, const char *cesa)
{
	i = 0;
	int nDigits = 0;
	bool negative = false;
	{int ch = GetCh(); if (ch == '-') negative = true; else PutBack(ch);}
	for ( ; ; )
	{
		int ch = GetCh();
		if (ch < 0) { if (nDigits <= 0) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Pricakoval bi stevilo, nasel sem EOF.\n", cesa, lineNo); return false; }
			else return true; }
		if (ch < int('0') || ch > int('9')) { PutBack(ch);
			if (nDigits <= 0) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Pricakoval bi stevilo, nasel sem znak 0x%02x.\n", cesa, lineNo, int(ch)); return false; }
			else return true; }
		// 2^31 = 2147483648
		//printf("i = %d\n", i);
		if (i > 214748364) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je preveliko (overflow).\n", cesa, lineNo); return false; }
		if (i < -214748364) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je premajhno (underflow).\n", cesa, lineNo); return false; }
		/*
		int j = 2147483647 - i, d = int(ch - '0');
		//printf("i = %d, j = %d, d = %d\n", i, j, d);
		if (j < d) { fprintf(fRept, "%d. stevilo je preveliko (overflow).\n", katero); return false; }
		*/
		int d = int(ch - '0');
		if (i == 214748364 && d > 7) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je preveliko (overflow).\n", cesa, lineNo); return false; }
		if (i == -214748364 && d > 8) { fprintf(fRept, "Napaka pri branju %s v %d. vrstici.  Stevilo je premajhno (underflow).\n", cesa, lineNo); return false; }
		i *= 10;
		if (negative) i -= d; else i += d;
		nDigits++;
	}
}


// > 1 --> error
// <= 0 --> OK (# points = return value + 10)
//
bool Test(const char* fnIn, const char* fnOut, const char* fnRept, const char* fnOrigOut, int& nPoints)
{
	enum { MaxPik = 9, MaxW = MaxPik + 2, MaxH = MaxPik + 1 };
	int w, h, maxPik, vhod[MaxH][MaxW];
	int izhod[2 * MaxH - 1][2 * MaxW - 1];
	int used[MaxPik + 1][MaxPik + 1];
	nPoints = 0;
	// Read the input file.
	FILE *fIn = fopen(fnIn, "rt"); assert(fIn);
	{int ok = fscanf(fIn, "%d\n", &maxPik); assert(ok == 1); 
	assert(3 <= maxPik); assert(maxPik <= MaxPik); w = maxPik + 2; h = maxPik + 1;
	for (int y = 0; y < h; y++) for (int x = 0; x < w; x++) {
		ok = fscanf(fIn, "%d", &vhod[y][x]); 
		assert(0 <= vhod[y][x]); assert(vhod[y][x] <= maxPik); }
	fclose(fIn);}
	// Read the output file.
	FILE *fRept = fopen(fnRept, "wt"); assert(fRept);
	FILE *fOut = fopen(fnOut, "rb"); if (! fOut) { fprintf(fRept, "Ne morem odpreti izhodne datoteke \"%s\".\n", fnOut ? fnOut : "\0"); fclose(fRept); return false; }
	Reader reader(fOut);
	int ww = 2 * w - 1, hh = 2 * h - 1;
	for (int y = 0; y < hh; y++) 
	{
		for (int x = 0; x < ww; x++)
		{
			int c = reader.GetCh();
			if (c == EOF) { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): nepricakovan konec datoteke.\n", y + 1, x + 1); fclose(fRept); return false; }
			if (! (c == '.' || c == '|' || c == '-' || ('0' <= c && c <= '9'))) { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): neveljaven znak (ni stevka, -, | ali pika).\n", y + 1, x + 1); fclose(fRept); return false; }
			izhod[y][x] = c;
		}
		if (y == hh - 1) {
			if (! reader.AtEof() && ! reader.AtEol()) { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi konec vrstice ali EOF.\n", y + 1, ww + 1); fclose(fRept); return false; }}
		else {
			if (! reader.AtEol()) { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi konec vrstice.\n", y + 1, ww + 1); fclose(fRept); return false; }}
		if (reader.AtEol()) { bool ok = reader.ReadLn(); assert(ok); }
	}
	if (! reader.AtEof()) { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi EOF.\n", hh + 1, 1); fclose(fRept); return false; }
	fclose(fOut);
	// Evaluate.
	const int DX[] = { 1, 0, -1, 0 }, DY[] = { 0, 1, 0, -1 };
	for (int yy = 0; yy < hh; yy++) for (int xx = 0; xx < ww; xx++)
	{
		if (yy % 2 == 0 && xx % 2 == 0)
		{
			int x = xx / 2, y = yy / 2;
			if (izhod[yy][xx] != '0' + vhod[y][x]) { fprintf(fRept, "Napaka v izhodni datoteki: izhodna mreza ni enaka vhodni.\n"); fclose(fRept); return false; }
		}
		else if (yy % 2 != 0 && xx % 2 != 0)
		{
			if (izhod[yy][xx] != '.') { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi piko.\n", yy + 1, xx + 1); fclose(fRept); return false; }
		}
		else if (yy % 2 != 0 && xx % 2 == 0)
		{
			if (izhod[yy][xx] != '.' && izhod[yy][xx] != '|') { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi | ali pa piko.\n", yy + 1, xx + 1); fclose(fRept); return false; }
		}
		else if (yy % 2 == 0 && xx % 2 != 0)
		{
			if (izhod[yy][xx] != '.' && izhod[yy][xx] != '-') { fprintf(fRept, "Napaka v izhodni datoteki (vrstica %d, stolpec %d): pricakoval bi - ali pa piko.\n", yy + 1, xx + 1); fclose(fRept); return false; }
		}
		else assert(false);
	}
	for (int x = 0; x <= maxPik; x++) for (int y = 0; y <= maxPik; y++) used[x][y] = 0;
	for (int y = 0; y < h; y++) for (int x = 0; x < w; x++)
	{
		int nNeigh = 0, xx = 2 * x, yy = 2 * y;
		int pik = izhod[yy][xx] - '0', neighPik = -1;
		for (int dir = 0; dir < 4; dir++)
		{
			int xx2 = xx + DX[dir], yy2 = yy + DY[dir];
			if (xx2 < 0 || xx2 >= ww || yy2 < 0 || yy2 >= hh) continue;
			if (izhod[yy2][xx2] == (DX[dir] == 0 ? '|' : '-')) 
			{
				int xx3 = xx2 + DX[dir], yy3 = yy2 + DY[dir];
				assert(xx3 >= 0 && xx3 < ww && yy3 >= 0 && yy3 < hh);
				assert(izhod[yy3][xx3] >= '0' && izhod[yy3][xx3] <= '0' + maxPik);
				nNeigh++; neighPik = izhod[yy3][xx3] - '0'; 
			}
		}
		if (nNeigh == 0) { fprintf(fRept, "Napaka v izhodni datoteki: nekatera polja ne pripadajo nobeni domini.\n"); fclose(fRept); return false; }
		if (nNeigh > 1) { fprintf(fRept, "Napaka v izhodni datoteki: nekatere domine pokrivajo vec kot 2 polji.\n"); fclose(fRept); return false; }
		assert(neighPik >= 0);
		if (neighPik < pik) { int t = pik; pik = neighPik; neighPik = t; }
		used[pik][neighPik]++;
	}
	for (int x = 0; x <= maxPik; x++) for (int y = x; y <= maxPik; y++)
	{
		assert(used[x][y] % 2 == 0);
		if (used[x][y] > 2) { fprintf(fRept, "Napaka v izhodni datoteki: nekje se pojavljata (vsaj) dve enaki domini.\n"); fclose(fRept); return false; }
	}
	nPoints = 10; 
	for (int x = 0; x <= maxPik; x++) for (int y = x; y <= maxPik; y++)
		if (used[x][y] == 0) { fprintf(fRept, "Resitev je videti pravilna, vendar niso uporabljene vse mozne domine.  Ocitno je v *vhodnih* podatkih neka napaka.\n"); fclose(fRept); return true; }
	fprintf(fRept, "OK.\n", nPoints); fclose(fRept); return true; 
}

int main(int argc, char** argv)
{
	assert(argc == 5);
	int nPoints = 0;
	bool ok = Test(argv[1], argv[2], argv[3], argv[4], nPoints);
	if (! ok) return -10; else return nPoints - 10;
}
